import io
import urllib
import urllib3

import pandas as pd

from marcuslion.config import api_version
from marcuslion.restcontroller import RestController


class Models(RestController):
    """
    MarcusLion Indicators class
    """

    def __init__(self):
        super().__init__(api_version + "/models")

    def list(self) -> pd.DataFrame:
        """
        Indicators.list()
        """
        return super().verify_get_df("", {})

    def query(self, ref):
        return super().verify_get_data("query", {"ref", ref})

    def search(self, search) -> pd.DataFrame:
        return super().verify_get_data("search", {"search", search})

    def download(self, ref, params) -> pd.DataFrame:
        """
        Indicators.download(ref, params)
        """
        pass

    def subscribe(self, ref, params):
        """
        Indicators.subscribe(ref, params)
        """
        pass



import io
import urllib
import urllib3

import pandas as pd

from marcuslion.config import base_url, api_key, api_version
from marcuslion.restcontroller import RestController


class Documents(RestController):
    """
    MarcusLion Documents class
    """

    def __init__(self):
        super().__init__(api_version + "/documents")

    def list(self) -> pd.DataFrame:
        """
        Documents.list()
        """
        return super().verify_get_data("", {})

    def search(self, search, provider_list) -> pd.DataFrame:
        """
        Documents.search(search)
        """
        params = {"providers": provider_list, "title": search}
        return super().verify_get_data("search", params)

    def query(self, ref):
        """
        Documents.query(ref)
        """
        pass

    def download(self, ref) -> pd.DataFrame:
        """
        Documents.download(ref)
        """
        pass



import pandas as pd

from marcuslion.config import api_version
from marcuslion.restcontroller import RestController


class DataFrames(RestController):
    """
    MarcusLion Datasets class
        https://qa1.marcuslion.com/swagger-ui/index.html#/data-frames-api-controller
    """

    def __init__(self):
        super().__init__(api_version + "/dataframes")

    def list(self) -> pd.DataFrame:
        return super().verify_get_data("", None)

    def search(self, search, provider_list) -> pd.DataFrame:
        params = {"providers": provider_list, "title": search}
        return super().verify_get_data("search", params)

    def query(self, _id) -> pd.DataFrame:
        return super().verify_get("metadata", {"id": _id})

    def download(self, _id) -> pd.DataFrame:
        return super().verify_get("download/" + _id, {})



import pandas as pd

from marcuslion.DataGatewayInterface import DataGatewayInterface
from marcuslion.config import api_version
from marcuslion.restcontroller import RestController


class UsSec(DataGatewayInterface):
    """
    MarcusLion UsSec
        https://qa1.marcuslion.com/swagger-ui/index.html#/sec-controller
    """
    def __init__(self):
        super().__init__(api_version + "/us-sec-edgar/company-facts")

    def list(self) -> pd.DataFrame:
        return super().verify_get_df()

    def query(self, cik) -> pd.DataFrame:
        return super().verify_get_df(f"{cik}")

    def get_fields(self, cik) -> any:
        return super().verify_get(f"{cik}")

    def get_units(self, cik, field) -> pd.DataFrame:
        return super().verify_get(f"{cik}/{field}")

    def search(self, search, provider_list) -> pd.DataFrame:
        # params = {"providers": provider_list, "title": search}
        # return super().verify_get_data("search", params)
        pass

    def download(self, cik) -> pd.DataFrame:
        pass



from marcuslion.restcontroller import RestController


class DataGatewayInterface(RestController):
    def __init__(self, url):
        super().__init__(url)

    def list(self):
        pass

    def search(self, query, params):
        pass

    def query(self, ident):
        pass

    def metadata(self, ident):
        pass

    def download(self, ident, params):
        pass

    def subscribe(self, ident, params):
        pass



import os

"""
MarcusLion Global variable
"""

api_key = os.getenv('MARCUSLION_API_KEY', "b7f88c9c-94ba-11ee-b9d1-0242ac120002")
base_url = os.getenv('MARCUSLION_API_HOST', "https://uat.marcuslion.com")
project_id = os.getenv('MARCUSLION_API_PROJECT_ID', "7ea7393b-017d-3ba9-a363-5b7a1043e0fes")
api_version = "core/api/v2"  # no starting slash



import pandas as pd

from marcuslion.config import api_version
from marcuslion.restcontroller import RestController


class Support(RestController):
    """
    MarcusLion Datasets class
        https://qa1.marcuslion.com/swagger-ui/index.html#/data-frames-api-controller
    """

    def __init__(self):
        super().__init__(api_version + "/support")  # /api/v2

    def standard_candles(self) -> pd.DataFrame:
        return super().verify_get_df("standardCandles", {})



import os
import pandas as pd

from marcuslion.config import api_version
from marcuslion.restcontroller import RestController


class Projects(RestController):
    """
    MarcusLion Projects class
        # $ curl 'https://qa1.marcuslion.com/core/projects'
    """

    def __init__(self, datasets):
        super().__init__(api_version + "/projects")
        self.datasets = datasets

    def list(self) -> pd.DataFrame:
        """
        Projects.list()
        """
        return super().verify_get_df("", {})

    def get_project_metadata(self, project_id: str) -> any:
        """
        Projects.get_project_metadata(id)
        """
        return super().verify_get(project_id)

    def __download_dataset_files(self, datasets: list, output_path: str = None) -> None:
        os.mkdir(output_path)

        # Download dataset files
        if datasets is None:
            return
        for dataset in datasets:
            try:
                self.datasets.candles(dataset["source"], dataset["refId"], dataset["file"], output_path)
            except ValueError as e:
                print(f"Failed to download dataset file {dataset['file']} for {dataset['refId']}: {e}")

    def load_project(self, project_id):
        """
        Projects.load_project(id)
        """
        try:
            metadata = self.get_project_metadata(project_id)
            print(f"Loading project {metadata['projectName']}")
            datasets = list(metadata['datasets'])

            self.__download_dataset_files(datasets, "./datasets")
        except ValueError as e:
            print(f"Failed to load project {project_id}: {e}")
